﻿Partial Public Class ControladorFiscal

    Public Enum ComandosFiscales As Integer

        ABRIR_COMPROBANTE_FISCAL = &H40
        IMPRIMIR_TEXTO_FISCAL = &H41
        IMPRIMIR_ITEM_FISCAL = &H42
        SUBTOTAL = &H43
        IMPRIMIR_PAGO = &H44
        CERRAR_COMPROBANTE_FISCAL = &H45

        ''' <summary>
        ''' Comandos utilizados por el protocolo para los impresores Epson,
        ''' a diferencia de la línea HASAR, no mantienen compatibilidad con los 
        ''' comandos entre distintos comprobantes.
        ''' </summary>
        ABRIR_COMPROBANTE_FISCAL_X = &H60
        IMPRIMIR_ITEM_FISCAL_X = &H62
        SUBTOTAL_X = &H63
        IMPRIMIR_PAGO_X = &H64
        CERRAR_COMPROBANTE_FISCAL_X = &H65

        ABRIR_COMPROBANTE_NO_FISCAL = &H48
        ABRIR_COMPROBANTE_NO_FISCAL_SLIP = &H47
        IMPRIMIR_TEXTO_NO_FISCAL = &H49
        CERRAR_COMPROBANTE_NO_FISCAL = &H4A

        CIERRE_DE_JORNADA_FISCAL = &H39
        ABRIR_GAVETA = &H7B
        DATOS_COMPRADOR = &H62

        ESTABLECER_FECHA_HORA = &H58
        OBTENER_FECHA_HORA = &H59

        ESTABLECER_ENCABEZADO_PIE = &H5D
        OBTENER_ENCABEZADO_PIE = &H5E

        ''' <summary>
        ''' Responde, a través de la línea serie, con el estado en que se encuentra el controlador fiscal y
        ''' el hardware de la impresora. No afecta la memoria de trabajo.
        ''' </summary>
        SOLICITAR_ESTADO = &H2A
        SOLICITAR_ESTADO_INTERMEDIO = &HA1

        ''' <summary>
        ''' Responde abriendo un documento no fiscal homologado e imprimiendo el encabezamiento,
        ''' incluyendo las leyendas referentes a los documentos no fiscales homologados.
        ''' </summary>
        ABRIR_DNFH = &H80
        CERRAR_DNFH = &H81

        ''' <summary>
        ''' Almacena en memoria de trabajo texto con la siguiente información:
        ''' a. Si el documento a abrir posteriormente es un tique factura o tique nota de débito: almacena
        '''    la identificación de un remito.
        ''' b. Si el documento a abrir posteriormente es una nota de crédito, u otro documento no fiscal
        '''    homologado: almacena el número del comprobante original que se vinculará al documento
        '''    a emitir. En este caso, la nota de crédito no podrá abrirse si antes no se ha emitido el 
        '''    presente comando.
        ''' </summary>
        ESTABLECER_NUMERO_EMBARQUE = 147
        OBTENER_NUMERO_EMBARQUE = 148

        ''' <summary>
        ''' Lectura del numero de firmware del controlador.
        ''' </summary>
        OBTENER_NUMERO_VERSION_FISCAL = &H7F

        ''' <summary>
        ''' Setear nueva velocidad de comunicación puerto serie del impresor.
        ''' </summary>
        ESTABLECER_VELOCIDAD_COMUNICACION = &HA0

        ESTABLECER_CONFIGURACION_GENERAL = &H95
        ''' <summary>
        ''' Responde, a través de la línea serie, con los parámetros de configuración con los que opera
        ''' el controlador fiscal. No afecta la memoria de trabajo.
        ''' </summary>
        OBTENER_CONFIGURACION_GENERAL = &H96

        ''' <summary>
        ''' Configura uno (a elección) de los parámetros de funcionamiento del controlador.
        ''' </summary>
        ESTABLECER_CONFIGURACION_DE_UNO = &H64

        ''' <summary>
        ''' Configura en bloque los parámetros de funcionamiento del controlador fiscal.
        ''' </summary>
        ESTABLECER_CONFIGURACION_EN_BLOQUE = &H65
        OBTENER_CONFIGURACION_EN_BLOQUE = &H66

        OBTENER_DATOS_INICIALIZACION = &H73

        CAMBIAR_RESPONSABILIDAD_IVA = &H63

        ''' <summary>
        ''' Cambia el número de inscripción en Ingresos Brutos, del emisor, almacenado en la memoria
        ''' EPROM fiscal. 
        ''' </summary>
        CAMBIAR_NUMERO_IB = &H6E


        ''' <summary>
        ''' Almacena en memoria de trabajo una línea de texto de la estructura FANTASY, la cual consta
        ''' de dos líneas que se imprimen automáticamente al comienzo del comprobante.
        ''' </summary>
        ESTABLECER_NOMBRE_FANTASIA = &H5F
        OBTENER_NOMBRE_FANTASIA = &H92

        ESTABLECER_LOGO = &H90
        RESETEAR_LOGO = &H91

        ''' <summary>
        ''' a. La cantidad total de registros diarios con que cuenta la memoria fiscal (3800 en modelos
        '''    SMH/P-715F, SMH/P-PR5F y SMH/P-441F; 1850 en el resto).
        ''' b. La cantidad de registros diarios que ya se han utilizados.
        '''    Restando la segunda cantidad de la primera se obtiene la cantidad de registros diarios libres.
        '''    Este comando no afecta la memoria de trabajo. 
        ''' </summary>
        OBTENER_CAPACIDAD_RESTANTE = &H37

        INFORME_FISCAL_POR_FECHA = &H3A

        ''' <summary>
        ''' Reporte de auditoria por número de ‘Z’
        ''' </summary>
        INFORME_FISCAL_POR_NUMERO = &H3B

        INFORME_DIARIO = &H3C
        IMFORME_MEMORIA_DE_TRABAJO = &H67

        OBTENER_PRIMER_DATO_DE_IVA = &H70
        OBTENER_SIGUIENTE_DATO_DE_IVA = &H71

        CANCELAR_DOCUMENTO = &H98

        REIMPRIMIR_ULTIMO_COMPROBANTE = &H99

        ESTABLECER_CODIGO_DE_BARRA = &H5A

        AVANZAR_PAPEL_TICKET = &H50
        AVANZAR_PAPEL_AUDITORIA = &H51

        AVANZAR_PAPEL_TICKET_Y_AUDITORIA = &H52

        DESCUENTO_ULTIMO_ITEM = &H55
        DESCUENTO_GENERAL = &H54

        EJECUTAR_SALTO_Y_TRANSPORTE = &H5C


    End Enum

    Public Enum SolicitudesDeEstado As Integer
        ''' <summary>
        ''' Información Normal o compatible con modelos anteriores.
        ''' </summary>
        NORMAL = 78
        ''' <summary>
        ''' Información sobre las Características del Controlador Fiscal.
        ''' </summary>
        CONTROLADOR_FISCAL = 80
        ''' <summary>
        ''' Información sobre el contribuyente.
        ''' </summary>
        CONTRIBUYENTE = 67
        ''' <summary>
        ''' Información sobre los contadores de documentos fiscales y no fiscales.
        ''' </summary>
        NUMERADORES = 65
        ''' <summary>
        ''' Información sobre el documento que se esta emitiendo.
        ''' </summary>
        DOCUMENTO_EN_CURSO = 68
        ''' <summary>
        ''' devuelve las preferencias del sistema
        ''' </summary>
        PREFERENCIAS_SISTEMA = 83

    End Enum

    Public Enum DocumentosFiscales As Integer

        NOTA_DEBITO_A = 68
        NOTA_DEBITO_B = 69
        RECIBO_A = 97
        RECIBO_B = 98
        TICKET_C = 84
        TICKET_FACTURA_A = 65
        TICKET_FACTURA_B = 66
        TICKET_NOTA_DEBITO_A = 50
        TICKET_NOTA_DEBITO_B = 51
        FACTURA_A = TICKET_FACTURA_A
        FACTURA_B = TICKET_FACTURA_B

        NOTA_CREDITO_A = 100
        NOTA_CREDITO_BC = 101
    End Enum

    Public Enum DocumentosNoFiscalesHomologados

        NOTA_CREDITO_A = 82
        NOTA_CREDITO_BC = 83
        TICKET_NOTA_CREDITO_A = NOTA_CREDITO_A
        TICKET_NOTA_CREDITO_BC = NOTA_CREDITO_BC
        
    End Enum



    Public Enum EstadosPapel

        SIN_PAPEL
        POCO_PAPEL
        NORMAL
    End Enum

    Public Enum ModelosFiscales

        EPSON_220
        EPSON_950
        EPSON_2002
        EPSON_LX300
        HASAR_441
        HASAR_715
        HASAR_715_VER1

    End Enum

    Public Enum EstadosFiscales
        ''' <summary>
        ''' Al encenderse la impresora se produjo un error en el checksum.
        ''' La impresora no funcionará
        ''' </summary>
        ERROR_CHEQUEO_MEMORIA = 0
        ERROR_MEMORIA_TRABAJO = 1
        ''' <summary>
        ''' La carga de la batería de respaldo de la memoria de trabajo se encuentra baja.
        ''' </summary>
        BATERIA_BAJA = 2
        ''' <summary>
        ''' El comando recibido no fue reconocido.
        ''' </summary>
        COMANDO_DESCONOCIDO = 3
        ''' <summary>
        ''' Uno de los campos del comando recibido tiene datos no válidos por ejemplo,
        ''' datos no numéricos en un campo numérico).
        ''' </summary>
        CAMPO_INVALIDO = 4
        ''' <summary>
        ''' Se ha recibido un comando que no es válido en el estado actual del controlador
        ''' (por ejemplo, abrir un recibo no-fiscal cuando se encuentra abierto un
        ''' recibo fiscal).
        ''' Nota: cuando se ha producido un cambio no válido de código de IVA, tanto el
        ''' bit 4 como el 5 tendrán valor 1.
        ''' </summary>
        COMANDO_INVALIDO_PARA_ESTADO_ACTUAL = 5
        DESBORDE_DE_TOTALES = 6
        MEMORIA_LLENA = 7
        MEMORIA_BAJA = 8
        TERMINA_CERTIFICADA = 9
        TERMINAL_FISCALIZADA = 10
        FECHA_INVALIDA = 11
        DOCUMENTO_FISCAL_ABIERTO = 12
        DOCUMENTO_ABIERTO = 13

    End Enum

    Public Enum EstadosImpresor
        ERROR_IMPRESORA = 2
        IMPRESORA_OFFLINE = 3
        SIN_PAPEL_AUDITORIA = 4
        SIN_PAPEL_TICKET = 5
        ''' <summary>
        ''' El controlador fiscal convierte los comandos enviados por un host en comandos
        ''' fiscales, y los almacena en un buffer antes de enviarlos a la impresora
        ''' fiscal. Cuando el buffer se aproxima a su capacidad máxima, este bit se
        ''' activa. Cualquier comando que se envíe cuando este bit está en 1 no se ejecuta
        ''' y debe ser reenviado por el host.
        ''' </summary>
        BUFFER_LLENO = 6
        BUFFER_VACIO = 7
        TAPA_ABIERTA = 8
        CAJON_DINERO_CERRADO = 14
    End Enum

    Public Enum ModosSecuencia
        AMBAS
        IMPAR
        PAR
    End Enum

    Public Enum ModosTrabajoLibreria

        ASINCRONO = 0
        POR_EVENTOS
        SINCRONO
    End Enum

    Public Enum ProtocolosComunicacion
        ACK
        BASE
        DTR
    End Enum

    Public Enum TiposDeDocumento As Integer

        TIPO_CI = 52
        TIPO_CUIL = 76
        TIPO_CUIT = 67
        TIPO_DNI = 50
        TIPO_LC = 49
        TIPO_LE = 48
        TIPO_NINGUNO = 32
        TIPO_PASAPORTE = 51

    End Enum

    Public Enum TiposDeResponsabilidades As Integer

        BIENES_DE_USO = 66
        CONSUMIDOR_FINAL = 67
        EVENTUAL = 86
        EVENTUAL_SOCIAL = 87
        MONOTRIBUTO = 77
        MONOTRIBUTO_SOCIAL = 83
        NO_CATEGORIZADO = 84
        NO_RESPONSABLE = 65
        RESPONSABLE_EXENTO = 69
        RESPONSABLE_INSCRIPTO = 73
        RESPONSABLE_NO_INSCRIPTO = 78
    End Enum

    Public Const F_ACK As Char = Chr(6)

    Public Const F_DC1 As Char = Chr(17)
    Public Const F_DC2 As Char = Chr(18)
    Public Const F_DC3 As Char = Chr(19)
    Public Const F_DC4 As Char = Chr(20)

    Public Const F_DEL As Char = Chr(127)
    Public Const F_ENQ As Char = Chr(5)
    Public Const F_EOT As Char = Chr(4)
    Public Const F_ESC As Char = Chr(27)
    Public Const F_ETX As Char = Chr(3)
    Public Const F_NAK As Char = Chr(21)
    Public Const F_SEP As Char = Chr(28)

    Public Const F_STX As Char = Chr(2)

    Public Const F_SEQUENCE_MAX As Integer = 120
    Public Const F_SEQUENCE_MIN As Integer = 32

    Public Const F_LEN_CRC As Integer = 4

End Class
