﻿
Public MustInherit Class ControladorFiscal

    ''' <summary>
    ''' comunicaciones por puerto serie!...
    ''' </summary>
    ''' <remarks></remarks>
    Private WithEvents m_fiscalPort As FiscalPort
    ''' <summary>
    ''' Timer uso interno para tratamiento de los distintos modos de trabajo.
    ''' </summary>
    ''' <remarks></remarks>
    Private m_fiscalTimer As FiscalTimer

    Private m_answer As String = String.Empty
    Private m_enteFiscal As EnteFiscal = Nothing
    Private m_escapeSequence As Boolean = False
    Private m_estadoPapel As EstadosPapel = EstadosPapel.NORMAL
    Private m_sequence As Integer = F_SEQUENCE_MIN
    Private m_modelo As ModelosFiscales
    Private m_modoTrabajo As ModosTrabajoLibreria = ModosTrabajoLibreria.SINCRONO

    Private m_protocoloComunicacion As ProtocolosComunicacion = ProtocolosComunicacion.BASE
    Private m_sequenceMode As ModosSecuencia = ModosSecuencia.AMBAS

    Private m_request As PaqueteFiscal
    Private m_response As PaqueteFiscal
    Private m_tipoResponsabilidadEmisor As TiposDeResponsabilidades = TiposDeResponsabilidades.RESPONSABLE_INSCRIPTO

    Private m_precioBase As Boolean
    Private m_documentoFiscal As DocumentosFiscales = DocumentosFiscales.TICKET_C

    Private m_logger As FiscalLogger = Nothing


    Public Property SecuenciaEscape() As Boolean
        Get
            Return m_escapeSequence
        End Get
        Set(ByVal value As Boolean)
            m_escapeSequence = value
        End Set
    End Property

    Friend Property DocumentoFiscal() As DocumentosFiscales
        Get
            Return m_documentoFiscal
        End Get
        Set(ByVal value As DocumentosFiscales)
            m_documentoFiscal = value
        End Set
    End Property

    Public Overridable Property EncabezadoPie(ByVal Linea As Integer) As String
        Get
            Try
                Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.OBTENER_ENCABEZADO_PIE)
                Return MyClass.EnviarYRecibir(pf.AgregarCampo(Linea, "00")).ParseString(3)
            Catch ex As Exception
                Throw New ExcepcionFiscal(ExcepcionFiscal.ExcepcionesFiscales.A_TRATAR_EN_APLICATIVO_CLIENTE, ex)
            End Try
        End Get
        Set(ByVal value As String)
            Try

                Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.ESTABLECER_ENCABEZADO_PIE)
                MyClass.Enviar(pf.AgregarCampo(Linea, "00").AgregarCampo(value))

            Catch ex As Exception
                Throw New ExcepcionFiscal(ExcepcionFiscal.ExcepcionesFiscales.A_TRATAR_EN_APLICATIVO_CLIENTE, ex)
            End Try

        End Set
    End Property

    Friend Property EnteFiscal() As EnteFiscal
        Get
            Return m_enteFiscal
        End Get
        Set(ByVal value As EnteFiscal)
            m_enteFiscal = value
        End Set
    End Property

    Public ReadOnly Property EstadoPapel() As EstadosPapel
        Get
            Return MyClass.m_estadoPapel
        End Get
    End Property

    Public Property FechaYHora() As DateTime
        Get
            Try
                Dim pf As PaqueteFiscal = MyClass.EnviarYRecibir(MyClass.CrearPaquete(ComandosFiscales.OBTENER_FECHA_HORA))
                Return FiscalUDFs.ToDateTime(pf.ParseString(3), pf.ParseString(4))
            Catch ex As Exception
                Throw New ExcepcionFiscal(ExcepcionFiscal.ExcepcionesFiscales.A_TRATAR_EN_APLICATIVO_CLIENTE, ex)
            End Try
        End Get
        Set(ByVal value As DateTime)
            Try
                Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.ESTABLECER_FECHA_HORA)

                pf.AgregarCampo(FiscalUDFs.ToFiscalDate(value))
                pf.AgregarCampo(FiscalUDFs.ToFiscalTime(value))

                MyClass.Enviar(pf)
            Catch ex As Exception
                Throw New ExcepcionFiscal(ExcepcionFiscal.ExcepcionesFiscales.A_TRATAR_EN_APLICATIVO_CLIENTE, ex)
            End Try
        End Set
    End Property

    Public ReadOnly Property HuboErrorFiscal() As Boolean
        Get
            Dim ans As Boolean = True
            If Not m_response Is Nothing Then
                Select Case True
                    Case m_response.EstadoFiscal(EstadosFiscales.CAMPO_INVALIDO)
                    Case m_response.EstadoFiscal(EstadosFiscales.COMANDO_DESCONOCIDO)
                    Case m_response.EstadoFiscal(EstadosFiscales.COMANDO_INVALIDO_PARA_ESTADO_ACTUAL)
                    Case m_response.EstadoFiscal(EstadosFiscales.DESBORDE_DE_TOTALES)
                    Case m_response.EstadoFiscal(EstadosFiscales.ERROR_CHEQUEO_MEMORIA)
                    Case m_response.EstadoFiscal(EstadosFiscales.ERROR_MEMORIA_TRABAJO)
                    Case m_response.EstadoFiscal(EstadosFiscales.FECHA_INVALIDA)
                    Case Else
                        ans = False
                End Select
            Else
                ans = False
            End If
            Return ans
        End Get
    End Property

    Public ReadOnly Property HuboErrorImpresor() As Boolean
        Get
            Dim ans As Boolean = True
            If Not m_response Is Nothing Then
                Select Case True
                    Case m_response.EstadoImpresor(EstadosImpresor.BUFFER_LLENO)
                    Case m_response.EstadoImpresor(EstadosImpresor.ERROR_IMPRESORA)
                    Case m_response.EstadoImpresor(EstadosImpresor.IMPRESORA_OFFLINE)
                    Case m_response.EstadoImpresor(EstadosImpresor.SIN_PAPEL_AUDITORIA)
                    Case m_response.EstadoImpresor(EstadosImpresor.SIN_PAPEL_TICKET)
                    Case m_response.EstadoImpresor(EstadosImpresor.TAPA_ABIERTA)
                    Case Else
                        ans = False
                End Select
            Else
                ans = False
            End If
            Return ans
        End Get
    End Property

    Friend ReadOnly Property Port() As FiscalPort
        Get
            If m_fiscalPort Is Nothing Then m_fiscalPort = New FiscalPort()
            Return m_fiscalPort
        End Get
    End Property

    Public ReadOnly Property Logger() As FiscalLogger
        Get
            If m_logger Is Nothing Then m_logger = New FiscalLogger()
            Return m_logger
        End Get
    End Property

    Public Property LoggerEnabled() As Boolean
        Get
            Return Logger.Enabled
        End Get
        Set(ByVal value As Boolean)
            LoggerEnabled("") = value
        End Set
    End Property

    Public Property LoggerEnabled(ByVal File As String) As Boolean
        Get
            Return Logger.Enabled
        End Get
        Set(ByVal value As Boolean)
            If Not m_logger Is Nothing Then m_logger.Enabled = False
            m_logger = New FiscalLogger(File)
            m_logger.Enabled = value
        End Set
    End Property

    Public Property Modelo() As ModelosFiscales
        Get
            Return m_modelo
        End Get
        Set(ByVal value As ModelosFiscales)
            m_modelo = value
        End Set
    End Property

    Friend Property ModoSecuencia() As ModosSecuencia
        Get
            Return m_sequenceMode
        End Get
        Set(ByVal value As ModosSecuencia)
            m_sequenceMode = value
        End Set
    End Property

    Public Property ModoTrabajoLibreria() As ModosTrabajoLibreria
        Get
            Return m_modoTrabajo
        End Get
        Set(ByVal value As ModosTrabajoLibreria)
            m_modoTrabajo = value
        End Set
    End Property

    Friend ReadOnly Property NextSequence() As Integer
        Get
            If m_sequence >= F_SEQUENCE_MAX Then
                m_sequence = F_SEQUENCE_MIN
            Else
                m_sequence += 1
            End If

            Select Case MyClass.ModoSecuencia
                Case ModosSecuencia.AMBAS 'do nothing!...
                Case ModosSecuencia.IMPAR : m_sequence += IIf(m_sequence Mod 2 = 0, 1, 0)
                Case ModosSecuencia.PAR : m_sequence += IIf(m_sequence Mod 2 = 0, 0, 1)
            End Select

            Return m_sequence
        End Get
    End Property

    Friend Property PrecioBase() As Boolean
        Get
            Return m_precioBase
        End Get
        Set(ByVal value As Boolean)
            m_precioBase = value
        End Set
    End Property

    Friend Property ProtocoloComunicacion() As ProtocolosComunicacion
        Get
            Return m_protocoloComunicacion
        End Get
        Set(ByVal value As ProtocolosComunicacion)
            m_protocoloComunicacion = value
        End Set
    End Property

    Friend ReadOnly Property Request() As PaqueteFiscal
        Get
            Return MyClass.m_request
        End Get
    End Property

    Friend ReadOnly Property Response() As PaqueteFiscal
        Get
            Return MyClass.m_response
        End Get
    End Property

    Public ReadOnly Property Respuesta() As PaqueteFiscal
        Get
            Return m_response
        End Get
    End Property

    Friend ReadOnly Property Timer() As FiscalTimer
        Get
            If m_fiscalTimer Is Nothing Then m_fiscalTimer = New FiscalTimer(1000)
            Return m_fiscalTimer
        End Get
    End Property

    ''' <summary>
    ''' Tipo de responsabilidad fiscal del emisor de comprobantes ....
    ''' </summary>
    Public Property TipoResponsabilidadEmisor() As TiposDeResponsabilidades
        Get
            Return m_tipoResponsabilidadEmisor
        End Get
        Set(ByVal value As TiposDeResponsabilidades)
            m_tipoResponsabilidadEmisor = value
        End Set
    End Property
    Public Property Timeout() As Double
        Get
            Return MyClass.Timer.Interval
        End Get
        Set(ByVal value As Double)
            MyClass.Timer.Interval = value
        End Set
    End Property

    Public Shared Function CrearInstancia(ByVal Modelo As ModelosFiscales) As ControladorFiscal
        Select Case True
            Case Modelo.ToString.Contains("HASAR")
                Return New ControladorFiscalHasar(Modelo)
            Case Modelo.ToString.Contains("EPSON")

                Select Case Modelo
                    Case ModelosFiscales.EPSON_2002 : Return New ControladorFiscalEpsonPrev(Modelo)
                    Case Else
                        Return New ControladorFiscalEpson(Modelo)
                End Select
            Case Else
                Return Nothing
        End Select
    End Function

    Private Sub m_fiscalPort_DataReceived(ByVal sender As Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles m_fiscalPort.DataReceived
        If MyClass.ModoTrabajoLibreria = ModosTrabajoLibreria.ASINCRONO Then
            MyClass.Recibir()
        End If
    End Sub

    Public Sub Comenzar()
        MyClass.Comenzar("COM1")
    End Sub

    Public Sub Comenzar(ByVal PortName As String)
        MyClass.Comenzar(PortName, ModosTrabajoLibreria.SINCRONO)
    End Sub

    Public Sub Comenzar(ByVal PortName As String, ByVal ModoTrabajo As ModosTrabajoLibreria)
        MyClass.ModoTrabajoLibreria = ModoTrabajo
        MyClass.Port.Open(PortName)
        MyClass.Port.DtrEnable = MyClass.ProtocoloComunicacion = ProtocolosComunicacion.DTR
    End Sub

    Public Function EnviarYRecibir(ByVal Comando As PaqueteFiscal) As PaqueteFiscal

        MyClass.Reset()
        MyClass.m_request = Comando
        MyClass.Port.Send(Comando.ToString(MyClass.SecuenciaEscape))

        Return MyClass.Recibir()
    End Function

    Public Function Enviar(ByVal Comando As PaqueteFiscal) As PaqueteFiscal


        MyClass.Reset()
        MyClass.m_request = Comando
        MyClass.Logger.writeLine("HOST   - " & Format(Now, "") & " : " & Comando.ToString(MyClass.SecuenciaEscape))
        MyClass.Port.Send(Comando.ToString(MyClass.SecuenciaEscape))

        If MyClass.ModoTrabajoLibreria = ModosTrabajoLibreria.SINCRONO Then Return Recibir()

        Return Nothing
    End Function

    Public Function CrearPaquete(ByVal Comando As ComandosFiscales)
        Return MyClass.CrearPaquete(Comando, MyClass.NextSequence)
    End Function

    Public Function CrearPaquete(ByVal Comando As ComandosFiscales, ByVal Secuencia As Integer) As PaqueteFiscal
        Return New PaqueteFiscal(Comando, Secuencia)
    End Function

    Public Function CrearPaquete(ByVal Comando As ComandosFiscales, _
                                   ByVal Secuencia As Integer, ByVal ParamArray Campos() As Object) As PaqueteFiscal
        Return New PaqueteFiscal(Comando, Secuencia, Campos)
    End Function

    Public Sub Finalizar()
        MyClass.Reset()
        MyClass.Port.Close()
        MyClass.Timer.Stop()
    End Sub

    Public Function Recibir() As PaqueteFiscal

        Dim buffer As String = String.Empty
        Dim input As String = String.Empty

        MyClass.Timer.Reset()

        Do While Not MyClass.Timer.IsExausted

            If MyClass.Port.CharsToRead > 0 Then

                input = MyClass.Port.Read(1)

                Select Case True

                    Case input = F_NAK : Throw New ExcepcionFiscal(ExcepcionFiscal.ExcepcionesFiscales.COMANDO_DESCONOCIDO)
                    Case input = F_ACK : If MyClass.ProtocoloComunicacion = ProtocolosComunicacion.ACK Then buffer = String.Empty
                    Case input = F_DC2 : MyClass.Timer.Reset() : buffer = String.Empty
                    Case input = F_DC4 : MyClass.m_estadoPapel = EstadosPapel.POCO_PAPEL
                    Case Else
                        buffer &= input

                        If FiscalUDFs.ValidateAnswer(buffer, MyClass.Request) Then
                            MyClass.m_response = FiscalUDFs.MakeFiscalResponse(FiscalUDFs.Answer, MyClass.Request)
                            MyClass.m_answer = MyClass.m_response.ToString(MyClass.SecuenciaEscape)
                            If MyClass.ProtocoloComunicacion = ProtocolosComunicacion.ACK Then
                                MyClass.Port.Send(F_ACK)
                            End If
                            Exit Do
                        End If

                End Select
            End If
        Loop

        If MyClass.Respuesta Is Nothing Then

            If MyClass.Timer.IsExausted Then Throw New ExcepcionFiscal(ExcepcionFiscal.ExcepcionesFiscales.SIN_CONTACTO_CON_EL_PRINTER)
            Return Nothing
        End If
        MyClass.Logger.writeLine("FISCAL - " & Format(Now, "") & " : " & MyClass.Respuesta.ToString(MyClass.SecuenciaEscape))
        Return MyClass.m_response
    End Function

    Public Sub Reset()
        If MyClass.ProtocoloComunicacion = ProtocolosComunicacion.ACK Then
            If MyClass.Port.CharsToRead > 0 Then
                MyClass.Port.Send(F_ACK) : MyClass.Port.ClearBuffers()
            End If
        ElseIf MyClass.Port.CharsToRead > 0 Then
            MyClass.Port.ClearBuffers()
        End If

        MyClass.m_estadoPapel = EstadosPapel.NORMAL
        MyClass.m_response = Nothing

    End Sub


    Public MustOverride Function AbrirGaveta() As PaqueteFiscal

    Public MustOverride Function DatosCliente(ByVal Nombre As String, _
                                              ByVal NumeroDocumento As String, _
                                              ByVal TipoDocumento As TiposDeDocumento, _
                                              ByVal ResponsabilidadIVA As TiposDeResponsabilidades) As PaqueteFiscal

    Public MustOverride Function DatosCliente(ByVal Nombre As String, _
                                              ByVal NumeroDocumento As String, _
                                              ByVal TipoDocumento As TiposDeDocumento, _
                                              ByVal ResponsabilidadIVA As TiposDeResponsabilidades, _
                                              ByVal Direccion As String) As PaqueteFiscal

    Public Function GenerarCoeficienteK(ByVal PrecioBase As Double, ByVal MontoImpuestoInterno As Double) As Double
        Dim ans As Double
        ans = PrecioBase / MontoImpuestoInterno
        ans = 1.0 / (1.0 + ans)
        Return ans
    End Function

    Public MustOverride Function ReporteX() As PaqueteFiscal
    Public MustOverride Function ReporteZ() As PaqueteFiscal
    Public MustOverride Function SolicitarEstado() As PaqueteFiscal
    Public MustOverride Function SolicitarEstado(ByVal Tipo As SolicitudesDeEstado) As PaqueteFiscal
    Public MustOverride Function AbrirComprobanteNoFiscal() As PaqueteFiscal
    Public MustOverride Function ImprimirTextoNoFiscal(ByVal Texto As String) As PaqueteFiscal
    Public MustOverride Function CerrarComprobanteNoFiscal() As PaqueteFiscal

    Public MustOverride Function AbrirComprobanteFiscal(ByVal Tipo As DocumentosFiscales) As PaqueteFiscal
    Public MustOverride Function ImprimirTextoFiscal(ByVal Texto As String) As PaqueteFiscal
    Public MustOverride Function ImprimirItem(ByVal Descripcion As String, _
                                              ByVal Cantidad As Double, _
                                              ByVal Monto As Double, _
                                              ByVal IVA As Double, _
                                              ByVal ImpuestosInternos As Double) As PaqueteFiscal

    Public MustOverride Function Subtotal() As PaqueteFiscal
    Public MustOverride Function Subtotal(ByVal Imprime As Boolean) As PaqueteFiscal
    Public MustOverride Function ImprimirPago(ByVal Descripcion As String, ByVal Monto As Double) As PaqueteFiscal
    Public MustOverride Function ImprimirPago(ByVal Descripcion As String, _
                                              ByVal Monto As Double, ByVal DescripcionAdicional As String) As PaqueteFiscal

    Public MustOverride Function CerrarComprobanteFiscal() As PaqueteFiscal
    Public MustOverride Function EstablecerNumeroEmbarque(ByVal Numero As String) As PaqueteFiscal
    Public MustOverride Function ObtenerNumeroEmbarque(ByVal Linea As Integer) As PaqueteFiscal
    Public MustOverride Function AbrirComprobanteNoFiscalHomologado(ByVal Tipo As DocumentosNoFiscalesHomologados) As PaqueteFiscal
    Public MustOverride Function CerrarComprobanteNoFiscalHomologado() As PaqueteFiscal


    ''' <summary>
    ''' Responde imprimiendo un reporte de auditoría entre Z_inicial / Z_final.
    ''' Si en la memoria fiscal existen registros correspondientes a tales números de ‘Z’, 
    ''' se informan los datos entre ellos.Los datos informados tienen las siguientes 
    ''' características, según sea el contenido del tercer campo:
    ''' a. verdadero : imprime los datos globales del rango.
    ''' b. falso     : imprime los datos del rango discriminados por jornada fiscal.
    ''' </summary>
    Public MustOverride Function ReporteZPorNumeros(ByVal ZetaInicial As Long, _
                                                    ByVal ZetaFinal As Long, ByVal DatosGlobales As Boolean) As PaqueteFiscal

    Public MustOverride Function DebeRealizarTransporte() As Boolean
    Public MustOverride Function RealizarTransporte() As PaqueteFiscal


End Class