﻿Imports LibreriaFiscal
Imports LibreriaFiscal.ControladorFiscal
Imports LibreriaFiscal.CampoFiscal

Public Class ControladorFiscalEpson : Inherits ControladorFiscal

    'utilizada para conservar las líneas de texto fiscal que se imprimen en las facturas A y B
    Protected m_fiscalLines() As String = Array.CreateInstance(GetType(String), 3)

    'utilizada por EstablecerNumeroEmbarque para aplicar a nro. remito.
    Protected m_remito As String = String.Empty

    Friend Sub New(ByVal Modelo As ModelosFiscales)


        For index As Integer = 0 To MyClass.m_fiscalLines.Length - 1
            MyClass.m_fiscalLines(index) = ControladorFiscal.F_DEL
        Next

        MyClass.Modelo = Modelo
        MyClass.ProtocoloComunicacion = ProtocolosComunicacion.DTR

        Select Case MyClass.Modelo
            Case ModelosFiscales.EPSON_220
                MyClass.SecuenciaEscape = False

            Case ModelosFiscales.EPSON_LX300

                MyClass.SecuenciaEscape = False

            Case Else
                'TODO!...
        End Select
    End Sub

    Public Overrides Property EncabezadoPie(ByVal Linea As Integer) As String
        Get
            Try
                Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.OBTENER_ENCABEZADO_PIE)
                Return MyClass.EnviarYRecibir(pf.AgregarCampo(Linea, "00")).ParseString(4)
            Catch ex As Exception
                Throw New ExcepcionFiscal(ExcepcionFiscal.ExcepcionesFiscales.A_TRATAR_EN_APLICATIVO_CLIENTE, ex)
            End Try
        End Get
        Set(ByVal value As String)
            MyBase.EncabezadoPie(Linea) = value
        End Set
    End Property

    Public Overrides Function AbrirGaveta() As PaqueteFiscal
        Return MyClass.Enviar(MyClass.CrearPaquete(ComandosFiscales.ABRIR_GAVETA))
    End Function

    Public Overrides Function DatosCliente(ByVal Nombre As String, _
                                           ByVal NumeroDocumento As String, _
                                           ByVal TipoDocumento As ControladorFiscal.TiposDeDocumento, _
                                           ByVal ResponsabilidadIVA As ControladorFiscal.TiposDeResponsabilidades) As PaqueteFiscal
        Return MyClass.DatosCliente(Nombre, NumeroDocumento, TipoDocumento, ResponsabilidadIVA, F_DEL)
    End Function

    Public Overrides Function DatosCliente(ByVal Nombre As String, _
                                           ByVal NumeroDocumento As String, _
                                           ByVal TipoDocumento As ControladorFiscal.TiposDeDocumento, _
                                           ByVal ResponsabilidadIVA As ControladorFiscal.TiposDeResponsabilidades, _
                                           ByVal Domicilio As String) As PaqueteFiscal
        MyBase.EnteFiscal = New EnteFiscal(Nombre, NumeroDocumento, TipoDocumento, ResponsabilidadIVA, Domicilio)
        Return Nothing
    End Function

    Public Overrides Function ReporteX() As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.CIERRE_DE_JORNADA_FISCAL)
        Return MyClass.Enviar(pf.AgregarCampo("X").AgregarCampo("P"))
    End Function

    Public Overrides Function ReporteZ() As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.CIERRE_DE_JORNADA_FISCAL)
        Return MyClass.Enviar(pf.AgregarCampo("Z").AgregarCampo("P"))
    End Function

    Public Overrides Function SolicitarEstado() As PaqueteFiscal
        Return MyClass.SolicitarEstado(SolicitudesDeEstado.NORMAL)
    End Function

    Public Overrides Function SolicitarEstado(ByVal Tipo As ControladorFiscal.SolicitudesDeEstado) As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.SOLICITAR_ESTADO)
        Return MyClass.Enviar(pf.AgregarCampo(Tipo, FormatosCampoFiscal.ASCII_TO_CHAR))
    End Function

    Public Overrides Function AbrirComprobanteNoFiscal() As PaqueteFiscal
        Return MyClass.Enviar(MyClass.CrearPaquete(ComandosFiscales.ABRIR_COMPROBANTE_NO_FISCAL))
    End Function

    Public Overrides Function ImprimirTextoNoFiscal(ByVal Texto As String) As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.IMPRIMIR_TEXTO_NO_FISCAL)
        Return MyClass.Enviar(pf.AgregarCampo(Texto))
    End Function

    Public Overrides Function CerrarComprobanteNoFiscal() As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.CERRAR_COMPROBANTE_NO_FISCAL)
        Return MyClass.Enviar(pf.AgregarCampo("T"))
    End Function

    Public Overrides Function AbrirComprobanteFiscal(ByVal Tipo As DocumentosFiscales) As PaqueteFiscal

        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.ABRIR_COMPROBANTE_FISCAL)

        'TODO: evaluar todos los casos posibles...

        MyClass.DocumentoFiscal = Tipo
        MyClass.PrecioBase = (MyClass.DocumentoFiscal = DocumentosFiscales.TICKET_FACTURA_A)

        Select Case MyClass.DocumentoFiscal
            Case DocumentosFiscales.TICKET_C : Return MyClass.Enviar(pf.AgregarCampo("C"))

            Case DocumentosFiscales.TICKET_FACTURA_A, _
                 DocumentosFiscales.TICKET_FACTURA_B, _
                 DocumentosFiscales.NOTA_CREDITO_A, DocumentosFiscales.NOTA_CREDITO_BC

                pf.Comando = ComandosFiscales.ABRIR_COMPROBANTE_FISCAL_X
                pf.AgregarCampo(EpsonDocumentosFiscales.DocumentoTipo(Tipo))
                pf.AgregarCampo("C")
                pf.AgregarCampo(EpsonDocumentosFiscales.Letra(Tipo))
                pf.AgregarCampo("1")
                pf.AgregarCampo("F")
                pf.AgregarCampo("12")
                pf.AgregarCampo(EpsonTiposDeResponsabilidad.Tipo(MyClass.TipoResponsabilidadEmisor))
                pf.AgregarCampo(EpsonTiposDeResponsabilidad.Tipo(MyClass.EnteFiscal.ResponsabilidadIVA))
                pf.AgregarCampo(MyClass.EnteFiscal.Nombre)
                pf.AgregarCampo(ControladorFiscal.F_DEL)
                pf.AgregarCampo(EpsonTipoDeDocumento.Tipo(MyClass.EnteFiscal.TipoDocumento))
                pf.AgregarCampo(MyClass.EnteFiscal.Documento)
                pf.AgregarCampo("N")
                pf.AgregarCampo(MyClass.EnteFiscal.Domicilio)
                pf.AgregarCampo(ControladorFiscal.F_DEL)
                pf.AgregarCampo(ControladorFiscal.F_DEL)

                pf.AgregarCampo(IIf(m_remito.Length > 0, m_remito, "R - 000000"))
                pf.AgregarCampo(ControladorFiscal.F_DEL)
                pf.AgregarCampo("C")

        End Select
        Return MyClass.Enviar(pf)
    End Function

    Public Overrides Function ImprimirTextoFiscal(ByVal Texto As String) As PaqueteFiscal

        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.IMPRIMIR_TEXTO_FISCAL)

        If MyClass.DocumentoFiscal = DocumentosFiscales.TICKET_C Then
            Return MyClass.Enviar(pf.AgregarCampo(Texto))
        Else
            For index As Integer = 0 To MyClass.m_fiscalLines.Length - 1
                If MyClass.m_fiscalLines(index) = ControladorFiscal.F_DEL Then
                    MyClass.m_fiscalLines(index) = Texto
                End If
            Next

        End If
        Return Nothing
    End Function

    Public Overrides Function ImprimirItem(ByVal Descripcion As String, _
                                           ByVal Cantidad As Double, _
                                           ByVal Monto As Double, _
                                           ByVal IVA As Double, _
                                           ByVal ImpuestosInternos As Double) As PaqueteFiscal

        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.IMPRIMIR_ITEM_FISCAL)
        Dim ii As Double = ImpuestosInternos

        Select Case MyClass.DocumentoFiscal
            Case DocumentosFiscales.TICKET_C

                pf.AgregarCampo(Descripcion)
                pf.AgregarCampo(Math.Abs(Cantidad), "00000.000", ".")
                pf.AgregarCampo(Math.Abs(Monto), "0000000.0000")
                pf.AgregarCampo(IVA, "00.00", ".")
                pf.AgregarCampo(IIf(Monto < 0, "m", "M"))
                pf.AgregarCampo("0")
                pf.AgregarCampo("0")
                pf.AgregarCampo(ImpuestosInternos, "0000000.00000000", ".")

            Case Else

                pf.Comando = ComandosFiscales.IMPRIMIR_ITEM_FISCAL_X
                pf.AgregarCampo(Descripcion)
                pf.AgregarCampo(Math.Abs(Cantidad), "00000.000", ".")
                pf.AgregarCampo(Math.Abs(Monto), "0000000.0000")
                pf.AgregarCampo(IVA, "00.00", ".")
                pf.AgregarCampo(IIf(Monto < 0, "m", "M"))
                pf.AgregarCampo("0")
                pf.AgregarCampo("0")

                For index As Integer = 0 To MyClass.m_fiscalLines.Length - 1
                    pf.AgregarCampo(MyClass.m_fiscalLines(index))
                Next

                pf.AgregarCampo(0.0, "00.00", ".")
                pf.AgregarCampo(ImpuestosInternos, "0000000.00000000", ".")

        End Select

        Return MyClass.Enviar(pf)
    End Function

    Public Overrides Function Subtotal() As PaqueteFiscal
        Return MyClass.Subtotal(False)
    End Function

    Public Overrides Function Subtotal(ByVal Imprime As Boolean) As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.SUBTOTAL_X)
        pf.AgregarCampo(IIf(Imprime, "P", "N")).AgregarCampo(ControladorFiscal.F_DEL)
        Return MyClass.Enviar(pf)
    End Function

    Public Overrides Function ImprimirPago(ByVal Descripcion As String, ByVal Monto As Double) As PaqueteFiscal
        Return MyClass.ImprimirPago(Descripcion, Monto, "")
    End Function

    Public Overrides Function ImprimirPago(ByVal Descripcion As String, _
                                           ByVal Monto As Double, ByVal DescripcionAdicional As String) As PaqueteFiscal

        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.IMPRIMIR_PAGO)

        If Not MyClass.DocumentoFiscal = DocumentosFiscales.TICKET_C Then pf.Comando = ComandosFiscales.IMPRIMIR_PAGO_X

        pf.AgregarCampo(Descripcion)
        pf.AgregarCampo(Math.Abs(Monto), "000000000.00", ".")
        pf.AgregarCampo(IIf(Monto < 0, "C", "T"))

        Return MyClass.Enviar(pf)

    End Function

    Public Overrides Function CerrarComprobanteFiscal() As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.CERRAR_COMPROBANTE_FISCAL)

        m_remito = String.Empty 'reinicializo para nuevo comprobante.

        If MyClass.DocumentoFiscal = DocumentosFiscales.TICKET_C Then
            Return MyClass.Enviar(pf.AgregarCampo("T"))
        End If
        pf.Comando = ComandosFiscales.CERRAR_COMPROBANTE_FISCAL_X
        pf.AgregarCampo(EpsonDocumentosFiscales.DocumentoTipo(MyClass.DocumentoFiscal))
        pf.AgregarCampo(EpsonDocumentosFiscales.Letra(MyClass.DocumentoFiscal))
        pf.AgregarCampo(ControladorFiscal.F_DEL)

        Return MyClass.Enviar(pf)
    End Function

    Public Overrides Function EstablecerNumeroEmbarque(ByVal Numero As String) As PaqueteFiscal
        m_remito = Numero
        Return Nothing
    End Function

    Public Overrides Function ObtenerNumeroEmbarque(ByVal Linea As Integer) As PaqueteFiscal
        Return Nothing 'TODO...
    End Function
    Public Overrides Function AbrirComprobanteNoFiscalHomologado(ByVal Tipo As ControladorFiscal.DocumentosNoFiscalesHomologados) As PaqueteFiscal
        Return Nothing 'TODO...
    End Function

    Public Overrides Function CerrarComprobanteNoFiscalHomologado() As PaqueteFiscal
        Return Nothing 'TODO...
    End Function

    Public Overrides Function ReporteZPorNumeros(ByVal ZetaInicial As Long, ByVal ZetaFinal As Long, ByVal DatosGlobales As Boolean) As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.INFORME_FISCAL_POR_NUMERO)
        pf.AgregarCampo(ZetaInicial).AgregarCampo(ZetaFinal)
        Return MyClass.Enviar(pf.AgregarCampo(IIf(DatosGlobales, "T", "d")))
    End Function

    Public Overrides Function DebeRealizarTransporte() As Boolean
        Return False
    End Function

    Public Overrides Function RealizarTransporte() As PaqueteFiscal
        Return Nothing
    End Function
End Class
