﻿Imports System.Text


Friend Class FiscalUDFs

    Private Shared m_answer As String = String.Empty


    Friend Shared ReadOnly Property Answer() As String
        Get
            Return m_answer
        End Get
    End Property

    Shared Function MakeFiscalCRC(ByVal Value As String) As String
        Return MakeFiscalCRC(Value, ControladorFiscal.F_LEN_CRC)
    End Function

    Shared Function MakeFiscalCRC(ByVal Value As String, ByVal Len As Integer) As String
        Dim ans As String = String.Empty
        Dim sum As Integer = 0
        For index As Integer = 0 To Value.Length - 1
            sum += Asc(Value.Substring(index, 1))
        Next
        Return PadLeft(Hex(sum), Len, "0").ToLower
    End Function

    Shared Function MakeFiscalResponse(ByVal Value As String, ByVal Request As PaqueteFiscal) As PaqueteFiscal

        Dim fs As String
        Dim pf As PaqueteFiscal = New PaqueteFiscal()
        Dim fields() As String = Value.Split(ControladorFiscal.F_SEP, ControladorFiscal.F_ETX)
        Dim position As Integer = IIf(Request.Packet.Contains(ControladorFiscal.F_ESC), 1, 0)
        For index As Integer = 0 To fields.Length - 1
            fs = fields(index)
            If (index = 0) Then
                pf.Comando = Asc(fs.Substring(fs.IndexOf(ControladorFiscal.F_STX) + 2 + position, 1))
                pf.Secuencia = Asc(fs.Substring(fs.IndexOf(ControladorFiscal.F_STX) + 1, 1))
            ElseIf (index = 1) Then ' printer status
                pf.AgregarEstadoImpresor(fs)
            ElseIf (index = 2) Then ' fiscal status
                pf.AgregarEstadoFiscal(fs)
            Else
                pf.AgregarCampo(fs)
            End If
        Next
        Return pf
    End Function


    ''' <summary>
    ''' Genera la lista de bytes con los estados del printer y el CF
    ''' </summary>
    Shared Function MakeStatusFields(ByVal value As String) As Boolean()
        Dim ans() As Boolean = Array.CreateInstance(GetType(Boolean), 16)
        Dim bytes As String = String.Empty
        For index As Integer = 0 To value.Length - 1
            bytes += PadLeft(Convert.ToString(CInt("&H" & value.Substring(index, 1)), 2), 4, "0")
        Next
        For index As Integer = 15 To 0 Step -1
            ans(15 - index) = bytes.Substring(index, 1) = "1"
        Next
        Return ans
    End Function

    Shared Function PadLeft(ByVal Value As String, ByVal Len As Integer) As String
        Return PadLeft(Value, Len, " ")
    End Function

    Shared Function PadLeft(ByVal Value As String, ByVal Len As Integer, ByVal Chars As String) As String
        If Value.Length >= Len Then Return Right(Value, Len)
        Return Right(Replicate(Chars, Len), Len - Value.Length) & Value
    End Function

    Shared Function Replicate(ByVal Value As String, ByVal Times As Integer) As String
        Return New StringBuilder().Append(Value, Times).ToString
    End Function

    Shared Function ToDate(ByVal fiscalDate As String) As Date
        Dim year As Integer = DateTime.Now.Year.ToString.Substring(0, 2) + fiscalDate.Substring(0, 2)
        Dim month As Integer = fiscalDate.Substring(2, 2)
        Dim day As Integer = fiscalDate.Substring(4, 2)
        Return New Date(year, month, day)
    End Function

    Shared Function ToDateTime(ByVal fiscalDate As String, ByVal fiscalTime As String) As DateTime

        Dim year As Integer = DateTime.Now.Year.ToString.Substring(0, 2) + fiscalDate.Substring(0, 2)
        Dim month As Integer = fiscalDate.Substring(2, 2)
        Dim day As Integer = fiscalDate.Substring(4, 2)

        Dim hour As Integer = fiscalTime.Substring(0, 2)
        Dim minute As Integer = fiscalTime.Substring(2, 2)
        Dim second As Integer = fiscalTime.Substring(4, 2)
        Return New DateTime(year, month, day, hour, minute, second)
    End Function

    Shared Function ToFiscalDate(ByVal value As DateTime) As String
        Dim ans As String = String.Empty
        ans += value.Year.ToString.Substring(2, 2)
        ans += value.Month.ToString("00")
        ans += value.Day.ToString("00")
        Return ans
    End Function

    Shared Function ToFiscalTime(ByVal value As DateTime) As String
        Dim ans As String = String.Empty
        ans += value.Hour.ToString("00")
        ans += value.Minute.ToString("00")
        ans += value.Second.ToString("00")
        Return ans
    End Function

    Shared Function ToTime(ByVal fiscalTime As String) As TimeSpan
        Dim hour As Integer = fiscalTime.Substring(0, 2)
        Dim minute As Integer = fiscalTime.Substring(2, 2)
        Dim second As Integer = fiscalTime.Substring(4, 2)
        Return New TimeSpan(hour, minute, second)
    End Function

    Shared Function ValidateAnswer(ByVal Answer As String, ByVal Request As PaqueteFiscal) As Boolean
        Dim ans As Boolean = False
        Dim pf As PaqueteFiscal = Nothing

        Dim body As String = Answer.Substring(0, Answer.IndexOf(ControladorFiscal.F_ETX) + 1)
        Dim crc As String = Right(Answer, Request.LenCRC)

        Select Case True
            Case Not Answer.Contains(ControladorFiscal.F_STX)
            Case Not Answer.Contains(ControladorFiscal.F_ETX)
            Case Not Answer.IndexOf(ControladorFiscal.F_STX) < Answer.IndexOf(ControladorFiscal.F_ETX)
            Case Not (Answer.Length - (Answer.IndexOf(ControladorFiscal.F_ETX) + 1)) >= Request.LenCRC
            Case Else

                m_answer = Answer.Substring(Answer.IndexOf(ControladorFiscal.F_STX))

                body = Answer.Substring(0, Answer.IndexOf(ControladorFiscal.F_ETX) + 1)
                crc = Right(Answer, Request.LenCRC)

                If FiscalUDFs.MakeFiscalCRC(body, Request.LenCRC) = crc.ToLower Then

                    m_answer = body
                    ans = True
                End If

        End Select

        Return ans
    End Function

    Shared Sub ShowFieldList(ByVal request As PaqueteFiscal)

        Dim s As StringBuilder = New StringBuilder(String.Empty)
        For index As Integer = 1 To request.Campos.Count
            s.Append(index.ToString("00") & " - " & request.Campos(index).ToString).Append(vbCrLf)
        Next

        MsgBox(s.ToString, MsgBoxStyle.Information, "Detalle request")

    End Sub
End Class
