﻿Imports LibreriaFiscalV2
Imports LibreriaFiscalV2.CampoFiscal

Public Class ControladorFiscalHasar : Inherits ControladorFiscal


    Friend Sub New(ByVal Modelo As ModelosFiscales)

        MyClass.Modelo = Modelo
        MyClass.ModoSecuencia = ModosSecuencia.IMPAR
        MyClass.ProtocoloComunicacion = ProtocolosComunicacion.ACK

        Select Case MyClass.Modelo
            Case ModelosFiscales.HASAR_441
                MyClass.SecuenciaEscape = True
            Case Else
                'TODO!...
        End Select
    End Sub

    Public Overrides Function AbrirGaveta() As PaqueteFiscal
        Return MyClass.Enviar(MyClass.CrearPaquete(ComandosFiscales.ABRIR_GAVETA))
    End Function

    Public Overrides Function DatosCliente(ByVal Nombre As String, _
                                           ByVal NumeroDocumento As String, _
                                           ByVal TipoDocumento As ControladorFiscal.TiposDeDocumento, _
                                           ByVal ResponsabilidadIVA As ControladorFiscal.TiposDeResponsabilidades) As PaqueteFiscal
        Return MyClass.DatosCliente(Nombre, NumeroDocumento, TipoDocumento, ResponsabilidadIVA, F_DEL)
    End Function

    Public Overrides Function DatosCliente(ByVal Nombre As String, _
                                           ByVal NumeroDocumento As String, _
                                           ByVal TipoDocumento As ControladorFiscal.TiposDeDocumento, _
                                           ByVal ResponsabilidadIVA As ControladorFiscal.TiposDeResponsabilidades, _
                                           ByVal Domicilio As String) As PaqueteFiscal
        Dim ef As EnteFiscal = New EnteFiscal(Nombre, NumeroDocumento, TipoDocumento, ResponsabilidadIVA, Domicilio)
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.DATOS_COMPRADOR)

        For Each campo As CampoFiscal In ef.ToFields
            pf.AgregarCampo(campo)
        Next
        Return Enviar(pf)

    End Function

    Public Overrides Function ReporteX() As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.CIERRE_DE_JORNADA_FISCAL)
        Return MyClass.Enviar(pf.AgregarCampo("X"))
    End Function

    Public Overrides Function ReporteZ() As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.CIERRE_DE_JORNADA_FISCAL)
        Return MyClass.Enviar(pf.AgregarCampo("Z"))
    End Function

    Public Overrides Function SolicitarEstado() As PaqueteFiscal
        Return MyClass.Enviar(MyClass.CrearPaquete(ComandosFiscales.SOLICITAR_ESTADO))
    End Function

    Public Overrides Function SolicitarEstado(ByVal Tipo As ControladorFiscal.SolicitudesDeEstado) As PaqueteFiscal
        Return MyClass.SolicitarEstado()
    End Function

    Public Overrides Function AbrirComprobanteNoFiscal() As PaqueteFiscal
        Return MyClass.Enviar(MyClass.CrearPaquete(ComandosFiscales.ABRIR_COMPROBANTE_NO_FISCAL))
    End Function

    Public Overrides Function ImprimirTextoNoFiscal(ByVal Texto As String) As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.IMPRIMIR_TEXTO_NO_FISCAL)
        Return MyClass.Enviar(pf.AgregarCampo(Texto).AgregarCampo("0"))
    End Function

    Public Overrides Function CerrarComprobanteNoFiscal() As PaqueteFiscal
        Return MyClass.Enviar(MyClass.CrearPaquete(ComandosFiscales.CERRAR_COMPROBANTE_NO_FISCAL))
    End Function

    Public Overrides Function AbrirComprobanteFiscal(ByVal Tipo As DocumentosFiscales) As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.ABRIR_COMPROBANTE_FISCAL)

        'TODO: evaluar todos los casos posibles...
        MyClass.PrecioBase = (Tipo = DocumentosFiscales.TICKET_FACTURA_A)

        Return MyClass.Enviar(pf.AgregarCampo(Tipo, FormatosCampoFiscal.ASCII_TO_CHAR).AgregarCampo("T"))
    End Function

    Public Overrides Function ImprimirTextoFiscal(ByVal Texto As String) As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.IMPRIMIR_TEXTO_FISCAL)
        Return MyClass.Enviar(pf.AgregarCampo(Texto).AgregarCampo("0"))
    End Function

    Public Overrides Function ImprimirItem(ByVal Descripcion As String, _
                                           ByVal Cantidad As Double, _
                                           ByVal Monto As Double, _
                                           ByVal IVA As Double, _
                                           ByVal ImpuestosInternos As Double) As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.IMPRIMIR_ITEM_FISCAL)
        Dim ii As Double = ImpuestosInternos

        If ImpuestosInternos > 0 Then
            If MyClass.PrecioBase Then
                ii = MyClass.GenerarCoeficienteK(Monto, ImpuestosInternos)
            Else
                ii = Monto - ImpuestosInternos
                ii = ii / (1 + (IVA / 100))
                ii = MyClass.GenerarCoeficienteK(ii, ImpuestosInternos)
            End If
        End If

        pf.AgregarCampo(Descripcion)
        pf.AgregarCampo(Cantidad, "000.0000000000")
        pf.AgregarCampo(Math.Abs(Monto), "000000.0000")
        pf.AgregarCampo(IVA, "00.00")
        pf.AgregarCampo(IIf(Monto >= 0.0, "M", "m"))
        pf.AgregarCampo(ii, "+0.00000000")
        pf.AgregarCampo("0").AgregarCampo(IIf(MyClass.PrecioBase, "B", "T"))

        Return MyClass.Enviar(pf)
    End Function

    Public Overrides Function Subtotal() As PaqueteFiscal
        Return MyClass.Subtotal(False)
    End Function

    Public Overrides Function Subtotal(ByVal Imprime As Boolean) As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.SUBTOTAL)
        pf.AgregarCampo(IIf(Imprime, "P", "N")).AgregarCampo("0")
        Return MyClass.Enviar(pf)
    End Function

    Public Overrides Function ImprimirPago(ByVal Descripcion As String, ByVal Monto As Double) As PaqueteFiscal
        Return MyClass.ImprimirPago(Descripcion, Monto, "")

    End Function

    Public Overrides Function ImprimirPago(ByVal Descripcion As String, _
                                           ByVal Monto As Double, ByVal DescripcionAdicional As String) As PaqueteFiscal

        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.IMPRIMIR_PAGO)
        pf.AgregarCampo(Descripcion)
        pf.AgregarCampo(Math.Abs(Monto))
        pf.AgregarCampo(IIf(Monto < 0, "C", "T")).AgregarCampo("0")

        Return MyClass.Enviar(pf)

    End Function

    Public Overrides Function CerrarComprobanteFiscal() As PaqueteFiscal
        Return MyClass.Enviar(MyClass.CrearPaquete(ComandosFiscales.CERRAR_COMPROBANTE_FISCAL))
    End Function

    Public Overrides Function EstablecerNumeroEmbarque(ByVal Numero As String) As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.ESTABLECER_NUMERO_EMBARQUE)
        Return MyClass.Enviar(pf.AgregarCampo("1").AgregarCampo(Numero))
    End Function

    Public Overrides Function ObtenerNumeroEmbarque(ByVal Linea As Integer) As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.ESTABLECER_NUMERO_EMBARQUE)
        Return MyClass.Enviar(pf.AgregarCampo(Linea.ToString))
    End Function

    Public Overrides Function AbrirComprobanteNoFiscalHomologado(ByVal Tipo As DocumentosNoFiscalesHomologados) As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.ABRIR_DNFH)
        Return MyClass.Enviar(pf.AgregarCampo(Tipo, FormatosCampoFiscal.ASCII_TO_CHAR).AgregarCampo("T"))
    End Function

    Public Overrides Function CerrarComprobanteNoFiscalHomologado() As PaqueteFiscal
        Return MyClass.Enviar(MyClass.CrearPaquete(ComandosFiscales.CERRAR_DNFH))
    End Function

    Public Overrides Function ReporteZPorNumeros(ByVal ZetaInicial As Long, ByVal ZetaFinal As Long, ByVal DatosGlobales As Boolean) As PaqueteFiscal
        Dim pf As PaqueteFiscal = MyClass.CrearPaquete(ComandosFiscales.INFORME_FISCAL_POR_NUMERO)
        pf.AgregarCampo(ZetaInicial).AgregarCampo(ZetaFinal)
        Return MyClass.Enviar(pf.AgregarCampo(IIf(DatosGlobales, "T", "N")))
    End Function

    Public Overrides Function DebeRealizarTransporte() As Boolean
        Return False
    End Function

    Public Overrides Function RealizarTransporte() As PaqueteFiscal
        Return Nothing
    End Function
End Class
