﻿Imports System.IO.Ports

Imports LibreriaFiscalV2.ExcepcionFiscal

Friend Class FiscalPort : Inherits SerialPort

    Enum BaudRates

        BAUD_2400 = 2400
        BAUD_4800 = 4800
        BAUD_9600 = 9600
        BAUD_19200 = 19200
        BAUD_38400 = 38400
        BAUD_57600 = 57600
        BAUD_115200 = 115200

    End Enum

    Enum PortBuffers
        ALL
        INPUT
        OUTPUT
    End Enum

    Private m_buffer As String = String.Empty
    Private m_dtrControl As Boolean = False

    ReadOnly Property CharsToRead() As Integer
        Get
            If Not MyBase.IsOpen Then Throw New ExcepcionFiscal(ExcepcionesFiscales.PUERTO_NO_DISPONIBLE)

            If MyBase.BytesToRead > 0 Then m_buffer &= MyBase.ReadExisting
            Return m_buffer.Length
        End Get
    End Property

    ReadOnly Property IndexOf(ByVal text As String) As Integer
        Get
            If MyClass.CharsToRead > 0 Then Return m_buffer.IndexOf(text)
        End Get
    End Property

    ReadOnly Property LastIndexOf(ByVal text As String) As Integer
        Get
            If MyClass.CharsToRead > 0 Then Return m_buffer.LastIndexOf(text)
        End Get
    End Property

    Public Overloads Property PortName() As String
        Get
            Return MyBase.PortName
        End Get
        Set(ByVal value As String)
            Dim abrir_luego As Boolean = MyBase.IsOpen
            Try
                If abrir_luego Then MyBase.Close()
                MyBase.PortName = value : If abrir_luego Then MyBase.Open()
            Catch ex As Exception
                Throw New ExcepcionFiscal(ExcepcionesFiscales.A_TRATAR_EN_APLICATIVO_CLIENTE, ex)
            End Try
        End Set
    End Property

    Overloads Sub Close()
        If MyBase.IsOpen Then MyBase.Close()
    End Sub

    Sub ClearBuffers()
        MyClass.ClearBuffers(PortBuffers.ALL)
    End Sub

    Sub ClearBuffers(ByVal buffer As PortBuffers)
        If Not MyBase.IsOpen Then Throw New ExcepcionFiscal(ExcepcionesFiscales.PUERTO_NO_DISPONIBLE)
        Select Case buffer
            Case PortBuffers.OUTPUT : MyBase.DiscardOutBuffer()
            Case PortBuffers.INPUT : m_buffer = String.Empty : MyBase.DiscardInBuffer()
            Case Else
                m_buffer = String.Empty : MyBase.DiscardInBuffer() : MyBase.DiscardOutBuffer()
        End Select
    End Sub

    Function Contains(ByVal text As String) As Boolean
        Return MyClass.IndexOf(text) >= 0
    End Function

    Overloads Sub Open()
        MyClass.Open("COM1")
    End Sub

    Overloads Sub Open(ByVal PortName As String)
        MyClass.Open(PortName, BaudRates.BAUD_9600)
    End Sub

    Overloads Sub Open(ByVal PortName As String, ByVal BaudRate As BaudRates)

        Try
            MyBase.Encoding = System.Text.Encoding.GetEncoding(1252)
            MyBase.PortName = PortName
            MyBase.BaudRate = BaudRate
            MyBase.DataBits = 8
            MyBase.StopBits = IO.Ports.StopBits.One
            'opcional para agrandar buffers de i/o
            '' MyBase.ReadBufferSize = 8192
            MyBase.WriteBufferSize = 8192
            MyBase.Open()

        Catch ex As Exception
            Throw New ExcepcionFiscal(ExcepcionesFiscales.A_TRATAR_EN_APLICATIVO_CLIENTE, ex)
        End Try

    End Sub

    Overloads Function Read(ByVal Chars As Integer) As String
        Dim ans As String = String.Empty
        If MyClass.CharsToRead() > 0 Then
            If m_buffer.Length >= Chars Then
                ans = m_buffer.Substring(0, Chars) : m_buffer = m_buffer.Substring(Chars)
            Else
                ans = m_buffer : m_buffer = String.Empty
            End If
        End If
        Return ans
    End Function

    Overloads Function ReadExisting() As String
        Dim ans As String = String.Empty
        If MyClass.CharsToRead() > 0 Then
            ans = m_buffer : m_buffer = String.Empty
        End If
        Return ans
    End Function

    Sub Send(ByVal Text As String)
        If Not MyBase.IsOpen Then Throw New ExcepcionFiscal(ExcepcionesFiscales.PUERTO_NO_DISPONIBLE)
        MyBase.Write(Text)
    End Sub


End Class
