﻿Imports System.Text

Imports LibreriaFiscalV2
Imports LibreriaFiscalV2.ControladorFiscal
Imports LibreriaFiscalV2.ExcepcionFiscal

Public Class PaqueteFiscal

    Private m_campos As Collection
    Private m_comando As ControladorFiscal.ComandosFiscales
    Private m_estadosFiscal() As Boolean
    Private m_estadosImpresor() As Boolean
    Private m_crc As String
    Private m_packet As String = String.Empty
    Private m_sequence As Integer

    ReadOnly Property Campos() As Collection
        Get
            If m_campos Is Nothing Then m_campos = New Collection
            Return m_campos
        End Get
    End Property

    ReadOnly Property Campo(ByVal Index As Integer) As CampoFiscal
        Get
            Try
                Return Campos.Item(Index)
            Catch ex As Exception
                Throw New ExcepcionFiscal(ExcepcionesFiscales.A_TRATAR_EN_APLICATIVO_CLIENTE, ex)
            End Try
        End Get
    End Property

    Public Property Comando() As ControladorFiscal.ComandosFiscales
        Get
            Return m_comando
        End Get
        Set(ByVal value As ControladorFiscal.ComandosFiscales)
            m_comando = value
        End Set
    End Property

    ReadOnly Property EstadoFiscal(ByVal Index As EstadosFiscales) As Boolean
        Get
            Try
                Return MyClass.m_estadosFiscal(Index)
            Catch ex As Exception
                Throw New ExcepcionFiscal(ExcepcionesFiscales.A_TRATAR_EN_APLICATIVO_CLIENTE, ex)
            End Try
        End Get
    End Property

    ReadOnly Property EstadoImpresor(ByVal Index As EstadosImpresor) As Boolean
        Get
            Try
                Return MyClass.m_estadosImpresor(Index)
            Catch ex As Exception
                Throw New ExcepcionFiscal(ExcepcionesFiscales.A_TRATAR_EN_APLICATIVO_CLIENTE, ex)
            End Try
        End Get
    End Property

    ReadOnly Property LenCRC() As Integer
        Get
            Return MyClass.m_crc.Length
        End Get
    End Property

    Friend ReadOnly Property Packet() As String
        Get
            Return m_packet
        End Get
    End Property

    Public Property Secuencia() As Integer
        Get
            Return m_sequence
        End Get
        Set(ByVal value As Integer)
            m_sequence = value
        End Set
    End Property

    Public Sub New()
    End Sub
    Public Sub New(ByVal Comando As ControladorFiscal.ComandosFiscales)
        MyClass.m_comando = Comando
    End Sub
    Public Sub New(ByVal Comando As ControladorFiscal.ComandosFiscales, ByVal Secuencia As Integer)
        MyClass.new(Comando)
        MyClass.m_sequence = Secuencia
    End Sub
    Public Sub New(ByVal Comando As ControladorFiscal.ComandosFiscales, ByVal Secuencia As Integer, _
                    ByVal ParamArray Campos() As Object)
        MyClass.new(Comando, Secuencia)
        For Each campo As Object In Campos
            If TypeOf campo Is CampoFiscal Then
                MyClass.AgregarCampo(campo)
            Else
                MyClass.AgregarCampo(New CampoFiscal(campo))
            End If
        Next
    End Sub

    Public Function AgregarCampo(ByVal Value As Object) As PaqueteFiscal
        Return MyClass.AgregarCampo(New CampoFiscal(Value))
    End Function

    Public Function AgregarCampo(ByVal Value As Object, ByVal Mask As String) As PaqueteFiscal
        Return MyClass.AgregarCampo(New CampoFiscal(Value, Mask))
    End Function

    Public Function AgregarCampo(ByVal Value As Object, ByVal Mask As String, ByVal SuppressPattern As String) As PaqueteFiscal
        Return MyClass.AgregarCampo(New CampoFiscal(Value, Mask, SuppressPattern))
    End Function
    Public Function AgregarCampo(ByVal Value As Object, ByVal Formato As CampoFiscal.FormatosCampoFiscal) As PaqueteFiscal
        Return MyClass.AgregarCampo(New CampoFiscal(Value, Formato))
    End Function

    Public Function AgregarCampo(ByVal Campo As CampoFiscal) As PaqueteFiscal
        MyClass.Campos.Add(Campo)
        Return Me
    End Function

    Function AgregarEstadoFiscal(ByVal Value As String) As PaqueteFiscal
        MyClass.AgregarCampo(Value)
        m_estadosFiscal = FiscalUDFs.MakeStatusFields(Value) : Return Me
    End Function

    Function AgregarEstadoImpresor(ByVal Value As String) As PaqueteFiscal
        MyClass.AgregarCampo(value)
        m_estadosImpresor = FiscalUDFs.MakeStatusFields(value) : Return Me
    End Function

    Public Function ParseDate(ByVal Campo As Integer) As Date
        Try
            Return FiscalUDFs.ToDate(MyClass.Campo(Campo).ToString)
        Catch ex As Exception
            Throw New ExcepcionFiscal(ExcepcionesFiscales.A_TRATAR_EN_APLICATIVO_CLIENTE, ex)
        End Try
    End Function

    Public Function ParseDouble(ByVal Campo As Integer) As Double
        Try
            Return Double.Parse(MyClass.Campos(Campo).ToString)
        Catch ex As Exception
            Throw New ExcepcionFiscal(ExcepcionesFiscales.A_TRATAR_EN_APLICATIVO_CLIENTE, ex)
        End Try
    End Function

    Public Function ParseInteger(ByVal Campo As Integer) As Integer
        Try
            Return Integer.Parse(MyClass.Campos(Campo).ToString)
        Catch ex As Exception
            Throw New ExcepcionFiscal(ExcepcionesFiscales.A_TRATAR_EN_APLICATIVO_CLIENTE, ex)
        End Try
    End Function

    Public Function ParseString(ByVal Campo As Integer) As String
        Try
            Return MyClass.Campo(Campo).ToString
        Catch ex As Exception
            Throw New ExcepcionFiscal(ExcepcionesFiscales.A_TRATAR_EN_APLICATIVO_CLIENTE, ex)
        End Try
    End Function

    Public Function ParseTime(ByVal Campo As Integer) As TimeSpan
        Try
            Return FiscalUDFs.ToTime(MyClass.Campo(Campo).ToString())
        Catch ex As Exception
            Throw New ExcepcionFiscal(ExcepcionesFiscales.A_TRATAR_EN_APLICATIVO_CLIENTE, ex)
        End Try
    End Function

    Public Overrides Function ToString() As String
        Return MyClass.ToString(False)
    End Function

    Public Overloads Function ToString(ByVal secuenciaEscape As Boolean) As String
        Dim ans As StringBuilder = New StringBuilder(String.Empty)

        ans.Append(ControladorFiscal.F_STX)
        ans.Append(Chr(MyClass.Secuencia))
        ans.Append(IIf(secuenciaEscape, ControladorFiscal.F_ESC, String.Empty))
        ans.Append(Chr(MyClass.Comando))

        For Each campo As CampoFiscal In MyClass.Campos
            ans.Append(ControladorFiscal.F_SEP).Append(campo.ToString)
        Next

        ans.Append(ControladorFiscal.F_ETX)

        MyClass.m_crc = FiscalUDFs.MakeFiscalCRC(ans.ToString)

        ans.Append(MyClass.m_crc)

        MyClass.m_packet = ans.ToString

        Return MyClass.m_packet

    End Function

End Class
